@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #include <libgnomevfs/gnome-vfs-utils.h>
@@ -34,7 +34,7 @@
 #include "cvs-interface.h"
 
 #define UI_FILE PACKAGE_DATA_DIR"/ui/anjuta-cvs.ui"
-#define ICON_FILE "anjuta-cvs-plugin.png"
+#define ICON_FILE "anjuta-cvs-plugin-48.png"
 
 static gpointer parent_class;
 
@@ -297,6 +297,9 @@ value_added_current_editor (AnjutaPlugin *plugin, const char *name,
 	
 	editor = g_value_get_object (value);
 	
+	if (!IANJUTA_IS_EDITOR(editor))
+		return;
+	
 	CVSPlugin *cvs_plugin = ANJUTA_PLUGIN_CVS (plugin);
 	ui = anjuta_shell_get_ui (plugin->shell, NULL);
 	
@@ -490,8 +493,7 @@ ipreferences_merge(IAnjutaPreferences* ipref, AnjutaPreferences* prefs, GError**
 static void
 ipreferences_unmerge(IAnjutaPreferences* ipref, AnjutaPreferences* prefs, GError** e)
 {
-	anjuta_preferences_dialog_remove_page(ANJUTA_PREFERENCES_DIALOG(prefs), 
-		_("CVS"));
+	anjuta_preferences_remove_page(prefs, _("CVS"));
 }
 
 static void
