@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 // TODO:
@@ -78,7 +78,7 @@ struct _CommonDebugTree {
 
 /* The debug tree object */
 struct _DebugTree {
-	IAnjutaDebugger *debugger;
+	DmaDebuggerQueue *debugger;
 	AnjutaPlugin *plugin;
 	GtkWidget* view;        /* the tree widget */
 	gboolean auto_expand;
@@ -95,7 +95,9 @@ struct _DmaVariablePacket {
 struct _DmaVariableData {
 	guchar modified;    /* Set by tree update */
 	guchar changed;     /* Set by global update */
-	gboolean analyzed;	/* Used as a tag to find variable outside scope */
+
+	gboolean exited;	/* variable outside scope */
+	gboolean deleted;	/* variable should be deleted */
 	
 	gboolean auto_update;
 	
@@ -169,7 +171,6 @@ dma_variable_data_new(const gchar *const name, gboolean auto_update)
 	}
 
 	data->changed = TRUE;
-	data->analyzed = TRUE;
 	data->auto_update = auto_update;
 		
 	return data;
@@ -332,7 +333,7 @@ delete_parent(GtkTreeModel *model, GtkTreePath* path,
 			{
 				/* Object has been created in debugger and is not a child
 			 	* (destroyed with their parent) */
-				ianjuta_variable_debugger_delete_var (IANJUTA_VARIABLE_DEBUGGER (tree->debugger), data->name, NULL);
+				dma_queue_delete_variable (tree->debugger, data->name);
 			}
 		}				
 
@@ -346,7 +347,7 @@ delete_parent(GtkTreeModel *model, GtkTreePath* path,
 
 
 static gboolean
-clear_analyzed(GtkTreeModel *model, GtkTreePath* path,
+set_deleted(GtkTreeModel *model, GtkTreePath* path,
 				 GtkTreeIter* iter, gpointer user_data)
 {
 	DmaVariableData *data;
@@ -356,8 +357,8 @@ clear_analyzed(GtkTreeModel *model, GtkTreePath* path,
 	
 	gtk_tree_model_get(model, iter, DTREE_ENTRY_COLUMN, &data, -1);
 	g_return_val_if_fail (data, TRUE);	/* Use on root node only, data != NULL */
-					 
-	data->analyzed = FALSE;
+
+	data->deleted= TRUE;
 
 	return FALSE;	
 }
@@ -411,7 +412,7 @@ debug_tree_add_children (DebugTree *tree, GtkTreeIter* parent, const GList *chil
 		
 		for (; child != NULL; child = g_list_next (child))
 		{
-			IAnjutaDebuggerVariable *var = (IAnjutaDebuggerVariable *)child->data;
+			IAnjutaDebuggerVariableObject *var = (IAnjutaDebuggerVariableObject *)child->data;
 			DmaVariableData *data;
 			
 			if (!valid_iter)
@@ -470,7 +471,7 @@ debug_tree_add_children (DebugTree *tree, GtkTreeIter* parent, const GList *chil
 }
 
 static void
-destroy_non_analyzed (DebugTree *tree, GtkTreeModel* model)
+destroy_deleted (DebugTree *tree, GtkTreeModel* model)
 {
 	DmaVariableData *data;
 	GtkTreeIter iter;
@@ -481,7 +482,7 @@ destroy_non_analyzed (DebugTree *tree, GtkTreeModel* model)
 	for (success = gtk_tree_model_get_iter_first (model, &iter); success == TRUE; )
 	{
 		gtk_tree_model_get(model, &iter, DTREE_ENTRY_COLUMN, &data, -1);
-		if ((data != NULL) && (data->analyzed == FALSE))
+		if ((data != NULL) && (data->deleted == TRUE))
 		{
 			success = debug_tree_remove (tree, &iter);
 		}
@@ -542,7 +543,7 @@ gdb_var_list_children (const GList *children, gpointer user_data, GError *err)
 }
 
 static void
-gdb_var_create (IAnjutaDebuggerVariable *variable, gpointer user_data, GError *err)
+gdb_var_create (IAnjutaDebuggerVariableObject *variable, gpointer user_data, GError *err)
 {
 	DmaVariablePacket *pack = (DmaVariablePacket *) user_data;
 	GtkTreeIter iter;
@@ -563,7 +564,7 @@ gdb_var_create (IAnjutaDebuggerVariable *variable, gpointer user_data, GError *e
 		 * created at this time, so remove it now */
 		if ((pack->tree->debugger) && (variable->name))
 		{
-			ianjuta_variable_debugger_delete_var (IANJUTA_VARIABLE_DEBUGGER (pack->tree->debugger), variable->name, NULL);
+			dma_queue_delete_variable (pack->tree->debugger, variable->name);
 		}
 		dma_variable_packet_free (pack);
 		
@@ -576,7 +577,6 @@ gdb_var_create (IAnjutaDebuggerVariable *variable, gpointer user_data, GError *e
 	{
 		data->name = strdup (variable->name);
 	}
-    data->analyzed = TRUE;
     data->changed = TRUE;
 	
 	gtk_tree_store_set(GTK_TREE_STORE(pack->model), &iter,
@@ -590,12 +590,11 @@ gdb_var_create (IAnjutaDebuggerVariable *variable, gpointer user_data, GError *e
 		DmaVariablePacket *pack_child =
                   dma_variable_packet_new(pack->model, &iter, pack->tree, data);
 
-		ianjuta_variable_debugger_list_children (
-				IANJUTA_VARIABLE_DEBUGGER (pack_child->tree->debugger),
+		dma_queue_list_children (
+				pack_child->tree->debugger,
 				variable->name,
 				(IAnjutaDebuggerCallback)gdb_var_list_children,
-				pack_child,
-				NULL);
+				pack_child);
 	}
 	else if (variable->children > 0)
 	{
@@ -611,12 +610,11 @@ gdb_var_create (IAnjutaDebuggerVariable *variable, gpointer user_data, GError *e
 	 * reusing same packet if possible */
 	if (variable->value == NULL)
 	{
-		ianjuta_variable_debugger_evaluate (
-				IANJUTA_VARIABLE_DEBUGGER (pack->tree->debugger),
+		dma_queue_evaluate_variable (
+				pack->tree->debugger,
 				variable->name,
 				(IAnjutaDebuggerCallback)gdb_var_evaluate_expression,
-				pack,
-				NULL);
+				pack);
 	}
 	else
 	{
@@ -643,12 +641,11 @@ on_treeview_row_expanded       (GtkTreeView     *treeview,
 		DmaVariablePacket *pack;
 					
 		pack = dma_variable_packet_new(model, iter, tree, data);
-		ianjuta_variable_debugger_list_children (
-						IANJUTA_VARIABLE_DEBUGGER (tree->debugger),
+		dma_queue_list_children (
+						tree->debugger,
 						data->name,
 						(IAnjutaDebuggerCallback)gdb_var_list_children,
-						pack,
-						NULL);
+						pack);
 	}
 	else
 	{
@@ -675,7 +672,7 @@ on_debug_tree_variable_changed (GtkCellRendererText *cell,
 		
 		if ((text != NULL) && (*text != '\0'))
 		{
-		    IAnjutaDebuggerVariable var = {NULL, NULL, NULL, NULL, FALSE, -1};
+		    IAnjutaDebuggerVariableObject var = {NULL, NULL, NULL, NULL, FALSE, FALSE, FALSE, -1};
 			
 			var.expression = text;
 			debug_tree_add_watch (tree, &var, TRUE);
@@ -704,14 +701,13 @@ on_debug_tree_value_changed (GtkCellRendererText *cell,
 		if (item != NULL)
 		{
 			/* Variable is valid */
-			ianjuta_variable_debugger_assign (IANJUTA_VARIABLE_DEBUGGER (tree->debugger), item->name, text, NULL);
+			dma_queue_assign_variable (tree->debugger, item->name, text);
 			tran = dma_variable_packet_new(model, &iter, tree, item);
-			ianjuta_variable_debugger_evaluate (
-					IANJUTA_VARIABLE_DEBUGGER (tree->debugger),
+			dma_queue_evaluate_variable (
+					tree->debugger,
 					item->name,
 					(IAnjutaDebuggerCallback) gdb_var_evaluate_expression,
-					tran,
-					NULL);
+					tran);
 		}
 	}
 }
@@ -869,12 +865,16 @@ debug_tree_find_expression (const GtkTreeModel *model, GtkTreeIter *iter, const
 	{
 		gchar *exp;
 		gchar *typ;
+		DmaVariableData *node;
 		gboolean found;
 		
-        gtk_tree_model_get (GTK_TREE_MODEL (model), iter, TYPE_COLUMN, &typ, VARIABLE_COLUMN, &exp, -1);
+        gtk_tree_model_get (GTK_TREE_MODEL (model), iter, TYPE_COLUMN, &typ,
+							VARIABLE_COLUMN, &exp,
+							DTREE_ENTRY_COLUMN, &node, -1);
 
 		found = ((type == NULL) || (strcmp (typ, type) == 0))
-			&& ((expression == NULL) || (strcmp (exp, expression) == 0));
+			&& ((expression == NULL) || (strcmp (exp, expression) == 0))
+			&& (node->exited == FALSE);
 		if (typ != NULL) g_free (typ);
 		if (exp != NULL) g_free (exp);
 		
@@ -890,16 +890,15 @@ on_replace_watch (gpointer data, gpointer user_data)
 	DebugTree* tree = (DebugTree *)user_data;
 	const gchar *expression = (const gchar *)data;
 	GtkTreeModel*const model = gtk_tree_view_get_model (GTK_TREE_VIEW(tree->view));
-	IAnjutaDebuggerVariable var = {NULL, NULL, NULL, NULL, FALSE, -1};
+	IAnjutaDebuggerVariableObject var = {NULL, NULL, NULL, NULL, FALSE, FALSE, FALSE, -1};
 	GtkTreeIter iter;
 
 	if (debug_tree_find_expression (model, &iter, expression, NULL))
 	{
 		DmaVariableData *data;
 
-        gtk_tree_model_get (model, &iter, DTREE_ENTRY_COLUMN, &data, -1);
-
-		if (data != NULL) data->analyzed = TRUE;
+        	gtk_tree_model_get (model, &iter, DTREE_ENTRY_COLUMN, &data, -1);
+		if (data != NULL) data->deleted = FALSE;
 	}
 	else
 	{
@@ -912,13 +911,13 @@ void
 debug_tree_replace_list (DebugTree *tree, const GList *expressions)
 {
 	GtkTreeModel* model = gtk_tree_view_get_model (GTK_TREE_VIEW(tree->view));
-		
-	/* Mark variables as not analyzed */
-	my_gtk_tree_model_foreach_child(model, NULL, clear_analyzed, NULL);	
+	
+	/* Mark variables as deleted */
+	my_gtk_tree_model_foreach_child(model, NULL, set_deleted, NULL);	
 
 	g_list_foreach ((GList *)expressions, on_replace_watch, tree);	
 
-	destroy_non_analyzed (tree, model);	
+	destroy_deleted (tree, model);	
 }
 
 void
@@ -940,7 +939,7 @@ debug_tree_add_dummy (DebugTree *tree, GtkTreeIter *parent)
  * variable object, currently only the expression field is set */
 
 void
-debug_tree_add_watch (DebugTree *tree, const IAnjutaDebuggerVariable* var, gboolean auto_update)
+debug_tree_add_watch (DebugTree *tree, const IAnjutaDebuggerVariableObject* var, gboolean auto_update)
 {
 	GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree->view));
 	GtkTreeIter iter;
@@ -968,12 +967,11 @@ debug_tree_add_watch (DebugTree *tree, const IAnjutaDebuggerVariable* var, gbool
 				DmaVariablePacket *pack;
 		
 				pack = dma_variable_packet_new(model, &iter, tree, data);
-				ianjuta_variable_debugger_create (
-						IANJUTA_VARIABLE_DEBUGGER (tree->debugger),
+				dma_queue_create_variable (
+						tree->debugger,
 						var->expression,
 						(IAnjutaDebuggerCallback)gdb_var_create,
-						pack,
-						NULL);
+						pack);
 			}
 			else
 			{
@@ -984,12 +982,11 @@ debug_tree_add_watch (DebugTree *tree, const IAnjutaDebuggerVariable* var, gbool
 					DmaVariablePacket *pack =
 					
 					pack = dma_variable_packet_new(model, &iter, tree, data);
-					ianjuta_variable_debugger_evaluate (
-							IANJUTA_VARIABLE_DEBUGGER (tree->debugger),
+					dma_queue_evaluate_variable (
+							tree->debugger,
 							var->name,
 							(IAnjutaDebuggerCallback)gdb_var_evaluate_expression,
-							pack,
-							NULL);
+							pack);
 				}
 				if (var->children == -1)
 				{
@@ -997,12 +994,11 @@ debug_tree_add_watch (DebugTree *tree, const IAnjutaDebuggerVariable* var, gbool
 					DmaVariablePacket *pack =
 					
 					pack = dma_variable_packet_new(model, &iter, tree, data);
-					ianjuta_variable_debugger_list_children (
-							IANJUTA_VARIABLE_DEBUGGER (tree->debugger),
+					dma_queue_list_children (
+							tree->debugger,
 							var->name,
 							(IAnjutaDebuggerCallback)gdb_var_list_children,
-							pack,
-							NULL);
+							pack);
 				}
 			}
 		}
@@ -1014,7 +1010,7 @@ on_add_watch (gpointer data, gpointer user_data)
 {
 	DebugTree* this = (DebugTree *)user_data;
 	gboolean auto_update = ((const gchar *)data)[0] & AUTO_UPDATE_WATCH ? TRUE : FALSE;
-	IAnjutaDebuggerVariable var = {NULL, NULL, NULL, NULL, FALSE, -1};
+	IAnjutaDebuggerVariableObject var = {NULL, NULL, NULL, NULL, FALSE, FALSE, FALSE, -1};
 
 	var.expression = &((gchar *)data)[1];
 	debug_tree_add_watch (this, &var, auto_update);
@@ -1030,7 +1026,7 @@ static void
 on_add_manual_watch (gpointer data, gpointer user_data)
 {
 	DebugTree* this = (DebugTree *)user_data;
-	IAnjutaDebuggerVariable var = {NULL, NULL, NULL, NULL, FALSE, -1};
+	IAnjutaDebuggerVariableObject var = {NULL, NULL, NULL, NULL, FALSE, FALSE, FALSE, -1};
 
 	var.expression = &((gchar *)data)[0];
 	debug_tree_add_watch (this, &var, FALSE);
@@ -1040,7 +1036,7 @@ static void
 on_add_auto_watch (gpointer data, gpointer user_data)
 {
 	DebugTree* this = (DebugTree *)user_data;
-	IAnjutaDebuggerVariable var = {NULL, NULL, NULL, NULL, FALSE, -1};
+	IAnjutaDebuggerVariableObject var = {NULL, NULL, NULL, NULL, FALSE, FALSE, -1};
 	
 	var.expression = &((gchar *)data)[0];
 	debug_tree_add_watch (this, &var, TRUE);
@@ -1055,7 +1051,7 @@ debug_tree_add_watch_list (DebugTree *this, GList *expressions, gboolean auto_up
 static void
 on_debug_tree_changed (gpointer data, gpointer user_data)
 {
-	IAnjutaDebuggerVariable *var = (IAnjutaDebuggerVariable *)data;
+	IAnjutaDebuggerVariableObject *var = (IAnjutaDebuggerVariableObject *)data;
 
 	if (var->name != NULL)
 	{
@@ -1074,7 +1070,12 @@ on_debug_tree_changed (gpointer data, gpointer user_data)
 				DmaVariableData *data;
 				gtk_tree_model_get (model, &iter, DTREE_ENTRY_COLUMN, &data, -1);
 		
-				if (data != NULL) data->changed = TRUE;
+				if (data != NULL)
+				{	
+					data->changed = var->changed;
+					data->exited = var->exited;
+					data->deleted = var->deleted;
+				}
 				
 				return;
 			}
@@ -1138,11 +1139,10 @@ debug_tree_update_all (DebugTree* tree)
 	if (tree->debugger != NULL)
 	{
 		/* Update if debugger is connected */
-		ianjuta_variable_debugger_update (
-				IANJUTA_VARIABLE_DEBUGGER (tree->debugger),
+		dma_queue_update_variable (
+				tree->debugger,
 				(IAnjutaDebuggerCallback)on_debug_tree_update_all,
-				tree,
-				NULL);
+				tree);
 	}
 }
 
@@ -1224,20 +1224,23 @@ debug_tree_update (DebugTree* tree, GtkTreeIter* iter, gboolean force)
 
 	if (data->name == NULL)
 	{
-		/* Variable need to be created first */
-		gchar *exp;
-		DmaVariablePacket *pack;
-			
-		gtk_tree_model_get (model, iter, VARIABLE_COLUMN, &exp, -1);
-		pack = dma_variable_packet_new(model, iter, tree, data);
-		data->modified = TRUE;
-		ianjuta_variable_debugger_create (
-				IANJUTA_VARIABLE_DEBUGGER (tree->debugger),
-				exp,
-				(IAnjutaDebuggerCallback)gdb_var_create,
-				pack,
-				NULL);
-		g_free (exp);
+		/* Check is the variable creation is not pending */
+		if (data->packet == NULL)
+		{
+			/* Variable need to be created first */
+			gchar *exp;
+			DmaVariablePacket *pack;
+		
+			gtk_tree_model_get (model, iter, VARIABLE_COLUMN, &exp, -1);
+			pack = dma_variable_packet_new(model, iter, tree, data);
+			data->modified = TRUE;
+			dma_queue_create_variable (
+					tree->debugger,
+					exp,
+					(IAnjutaDebuggerCallback)gdb_var_create,
+					pack);
+			g_free (exp);
+		}
 		
 		return FALSE;
 	}	
@@ -1246,12 +1249,11 @@ debug_tree_update (DebugTree* tree, GtkTreeIter* iter, gboolean force)
 		DmaVariablePacket *pack = dma_variable_packet_new(model, iter, tree, data);
 		refresh = data->modified != (data->changed != FALSE);
 		data->modified = (data->changed != FALSE);
-		ianjuta_variable_debugger_evaluate (
-				IANJUTA_VARIABLE_DEBUGGER (tree->debugger),
+		dma_queue_evaluate_variable (
+				tree->debugger,
 				data->name,
 				(IAnjutaDebuggerCallback)gdb_var_evaluate_expression,
-				pack,
-				NULL);
+				pack);
 	}
 	else
 	{
@@ -1316,7 +1318,7 @@ debug_tree_get_auto_update (DebugTree* this, GtkTreeIter* iter)
 }
 
 void
-debug_tree_connect (DebugTree *this, IAnjutaDebugger* debugger)
+debug_tree_connect (DebugTree *this, DmaDebuggerQueue* debugger)
 {
 	this->debugger = debugger;
 }
@@ -1360,6 +1362,72 @@ debug_tree_remove_model (DebugTree *tree, GtkTreeModel *model)
 	gtk_tree_store_clear (GTK_TREE_STORE (model));
 }
 
+gchar *
+debug_tree_get_selected (DebugTree *tree)
+{
+	GtkTreeIter iter;
+	gchar *exp = NULL;
+
+	if (get_current_iter (GTK_TREE_VIEW (tree->view), &iter))
+	{
+		GtkTreeModel *model;
+
+		model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree->view));
+		if (model != NULL)
+		{
+			gtk_tree_model_get(model, &iter, VARIABLE_COLUMN, &exp, -1);
+		}
+	}
+	
+	return exp;
+}
+
+gchar *
+debug_tree_get_first (DebugTree *tree)
+{
+	GtkTreeIter iter;
+	gchar *exp = NULL;
+	GtkTreeModel *model;
+	
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree->view));
+	if (model != NULL)
+	{
+		if (gtk_tree_model_get_iter_first (model, &iter))
+		{
+			gtk_tree_model_get(model, &iter, VARIABLE_COLUMN, &exp, -1);
+		}
+	}
+	
+	return exp;
+}
+
+gchar*
+debug_tree_find_variable_value (DebugTree *tree, const gchar *name)
+{
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree->view));
+	
+	if (gtk_tree_model_get_iter_first (model, &iter))
+	{
+		gchar *exp;
+		gchar *value;
+		do
+		{
+			gtk_tree_model_get(model, &iter, VARIABLE_COLUMN, &exp, 
+							                 VALUE_COLUMN, &value, -1);
+			
+			if (strcmp (exp, name) == 0)
+			{
+				return value;
+			}
+		} while (gtk_tree_model_iter_next (model, &iter));
+	}
+	
+	return NULL;
+}
+
 /* Constructor & Destructor
  *---------------------------------------------------------------------------*/
 
@@ -1398,7 +1466,6 @@ void
 debug_tree_free (DebugTree * tree)
 {
 	GtkTreeModel *model;
-	// AnjutaUI *ui;
 	
 	g_return_if_fail (tree);
 
