@@ -26,7 +26,6 @@
 #include <libanjuta/anjuta-preferences.h>
 #include <libgnomevfs/gnome-vfs-monitor.h>
 
-#include "text_editor_menu.h"
 #include "aneditor.h"
 
 #include "tm_tagmanager.h"
@@ -112,6 +111,8 @@ struct _TextEditor
 	 */
 	gint file_modified_timer;
 	GtkWidget *file_modified_widget;
+	
+	gboolean hover_tip_on;
 };
 
 struct _TextEditorClass
@@ -174,6 +175,7 @@ void text_editor_replace_selection (TextEditor * te, const gchar * r_str);
 guint    text_editor_get_total_lines (TextEditor * te);
 glong    text_editor_get_current_position (TextEditor * te);
 guint    text_editor_get_current_lineno (TextEditor * te);
+guint    text_editor_get_position_lineno (TextEditor * te, gint position);
 guint    text_editor_get_current_column (TextEditor * te);
 guint    text_editor_get_line_from_position (TextEditor * te, glong pos);
 gchar*   text_editor_get_selection (TextEditor * te);
@@ -239,6 +241,10 @@ void text_editor_set_busy (TextEditor *te, gboolean state);
 void text_editor_add_view (TextEditor *te);
 void text_editor_remove_view (TextEditor *te);
 
+/* Show/hide hover tips */
+void text_editor_show_hover_tip (TextEditor *te, gint position, const gchar *info);
+void text_editor_hide_hover_tip (TextEditor *te);
+
 /* Direct editor commands to AnEditor and Scintilla */
 void text_editor_command(TextEditor *te, gint command,
 						 glong wparam, glong lparam);
