@@ -86,27 +86,6 @@ on_text_editor_text_event (GtkWidget * widget,
 	return TRUE;
 }
 
-void
-on_text_editor_text_changed (GtkEditable * editable, gpointer user_data)
-{
-	TextEditor *te = TEXT_EDITOR (user_data);
-	if (text_editor_is_saved (te))
-	{
-		//FIXME:
-		// anjuta_update_title ();
-		// update_main_menubar ();
-		text_editor_update_controls (te);
-	}
-}
-
-void
-on_text_editor_insert_text (GtkEditable * text,
-			    const gchar * insertion_text,
-			    gint length, gint * pos, TextEditor * te)
-{
-}
-
-
 static void 
 scintilla_uri_dropped (TextEditor *te, const char *uri)
 {
@@ -173,11 +152,14 @@ on_text_editor_scintilla_notify (GtkWidget * sci, gint wParam, gpointer lParam,
 		return;
 		
 	case SCN_CHARADDED:
-		te->current_line = text_editor_get_current_lineno (te);
-		position = text_editor_get_current_position (te) - 1;
-		g_signal_emit_by_name(G_OBJECT (te), "char_added", position,
-							  (gchar)nt->ch);
-		
+		{
+			position = text_editor_get_current_position (te) - 1;
+			TextEditorCell *position_iter = text_editor_cell_new (te, position);
+			te->current_line = text_editor_get_current_lineno (te);
+			g_signal_emit_by_name(G_OBJECT (te), "char-added", position_iter,
+								  (gchar)nt->ch);
+			g_object_unref (position_iter);
+		}
 		return;
 	case SCN_AUTOCSELECTION:
 	case SCN_USERLISTSELECTION:
@@ -191,10 +173,13 @@ on_text_editor_scintilla_notify (GtkWidget * sci, gint wParam, gpointer lParam,
 	case SCN_MODIFIED:
 		if (nt->modificationType & (SC_MOD_INSERTTEXT | SC_MOD_DELETETEXT))
 		{
+			TextEditorCell *position_iter =
+					text_editor_cell_new (te, nt->position);
 			gboolean added = nt->modificationType & SC_MOD_INSERTTEXT;
-			g_signal_emit_by_name (G_OBJECT (te), "changed", nt->position,
+			g_signal_emit_by_name (G_OBJECT (te), "changed", position_iter,
 								   added, nt->length, nt->linesAdded,
 								   nt->text);
+			g_object_unref (position_iter);
 		}
 		return;
 	case SCN_MARGINCLICK:
