@@ -17,8 +17,8 @@
  * You should have received a copy of the GNU General Public License
  * along with plugin.c.  See the file "COPYING".  If not,
  * write to:  The Free Software Foundation, Inc.,
- *            59 Temple Place - Suite 330,
- *            Boston,  MA  02111-1307, USA.
+ *            51 Franklin Street, Fifth Floor,
+ *            Boston,  MA  02110-1301, USA.
  */
 
 #include "gprof-view.h"
@@ -117,7 +117,7 @@ gprof_view_show_symbol_in_editor (GProfView *self,
 {
 	IAnjutaIterable *symbol_iter;
 	IAnjutaSymbol *symbol;
-	const gchar *file;
+	const gchar *uri;
 	guint line;
 	
 	if (self->priv->symbol_manager &&
@@ -134,11 +134,11 @@ gprof_view_show_symbol_in_editor (GProfView *self,
 			ianjuta_iterable_get_length (symbol_iter, NULL) > 0)
 		{
 			symbol = IANJUTA_SYMBOL (symbol_iter);
-			file = ianjuta_symbol_file (symbol, NULL);
+			uri = ianjuta_symbol_uri (symbol, NULL);
 			line = ianjuta_symbol_line (symbol, NULL);
 			
-			ianjuta_document_manager_goto_file_line (self->priv->document_manager, 
-													 file, line, NULL);
+			ianjuta_document_manager_goto_uri_line (self->priv->document_manager, 
+													 uri, line, NULL);
 			
 			g_object_unref (symbol_iter);
 		}
@@ -158,3 +158,21 @@ gprof_view_get_widget (GProfView *self)
 {
 	return GPROF_VIEW_GET_CLASS (self)->get_widget (self);
 }
+
+void 
+gprof_view_format_float (GtkTreeViewColumn *col,  GtkCellRenderer *renderer,
+						 GtkTreeModel *model, GtkTreeIter *iter,
+					     gpointer column_number)
+{
+	gfloat number;
+	gchar *formatted_number;
+	
+	gtk_tree_model_get (model, iter, GPOINTER_TO_INT (column_number), &number,
+						-1);
+	
+	formatted_number = g_strdup_printf ("%0.2f", number);
+	g_object_set (renderer, "text", formatted_number, NULL);
+	
+	g_free (formatted_number);
+}
+
