@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #include <config.h>
@@ -28,7 +28,7 @@
 #include "plugin.h"
 #include "project-import.h"
 
-#define ICON_FILE "anjuta-project-import-plugin.png"
+#define ICON_FILE "anjuta-project-import-plugin-48.png"
 
 static gpointer parent_class;
 
