@@ -577,27 +577,33 @@ confirm_removal (ProjectManagerPlugin *plugin, GbfTreeData *data)
 {
 	gboolean answer;
 	gchar *mesg;
+	gchar* question;
+	gchar* full_mesg;
 
 	switch (data->type)
 	{
 		case GBF_TREE_NODE_GROUP:
-			mesg = _("%sGroup: %s\n\nThe group will not be deleted from file system.");
+			question = _("Are you sure you want to remove the following group from project?\n\n");
+			mesg = _("Group: %s\n\nThe group will not be deleted from file system.");
 			break;
 		case GBF_TREE_NODE_TARGET:
-			mesg = _("%sTarget: %s");
+			question = _("Are you sure you want to remove the following target from project?\n\n");
+			mesg = _("Target: %s");
 			break;
 		case GBF_TREE_NODE_TARGET_SOURCE:
-			mesg = _("%sSource: %s\n\nThe source file will not be deleted from file system.");
+			question = _("Are you sure you want to remove the following source file from project?\n\n");
+			mesg = _("Source: %s\n\nThe source file will not be deleted from file system.");
 			break;
 		default:
-			g_warning ("Unknow node");
+			g_warning ("Unknown node");
 			return FALSE;
 	}
+	full_mesg = g_strconcat (question, mesg, NULL);
 	answer =
 		anjuta_util_dialog_boolean_question (get_plugin_parent_window (plugin),
-											 mesg,
-		_("Are you sure you want to remove the following from project?\n\n"),
+											 full_mesg, question,
 											 data->name);
+	g_free (full_mesg);
 	return answer;
 }
 
@@ -698,9 +704,9 @@ on_popup_add_to_project (GtkAction *action, ProjectManagerPlugin *plugin)
 		if (res != GNOME_VFS_OK)
 			mesg = gnome_vfs_result_to_string (res);
 		else
-			mesg = _("URI is link");
+			mesg = _("The file you selected is a link and can't be added to the project");
 		anjuta_util_dialog_error (win,
-								  _("Failed to retried URI info of %s: %s"),
+								  _("Failed to retrieve URI info of %s: %s"),
 								  plugin->fm_current_uri, mesg);
 	}
 }
@@ -1212,8 +1218,7 @@ project_manager_unload_gbf (ProjectManagerPlugin *pm_plugin)
 			GList *node;
 			
 			editors =
-				ianjuta_document_manager_get_documents (IANJUTA_DOCUMENT_MANAGER
-													  (docman), NULL);
+				ianjuta_document_manager_get_doc_widgets (docman, NULL);
 			node = editors;
 			while (node)
 			{
@@ -2224,7 +2229,7 @@ iproject_manager_iface_init(IAnjutaProjectManagerIface *iface)
 	iface->get_selected = iproject_manager_get_selected;
 	iface->get_capabilities = iproject_manager_get_capabilities;
 	iface->add_source = iproject_manager_add_source;
-	iface->add_source_multi = iproject_manager_add_source_multi;
+	iface->add_sources = iproject_manager_add_source_multi;
 	iface->add_target = iproject_manager_add_target;
 	iface->add_group = iproject_manager_add_group;
 	iface->is_open = iproject_manager_is_open;
@@ -2249,7 +2254,9 @@ ifile_open (IAnjutaFile *ifile, const gchar* uri, GError **e)
 	/* If there is already a project loaded, load in separate anjuta window */
 	if (plugin->project_root_uri)
 	{
-		gchar *cmd = g_strconcat ("anjuta --no-splash --no-client ", uri, NULL);
+		gchar *quoted_uri = g_shell_quote (uri);
+		gchar *cmd = g_strconcat ("anjuta --no-splash --no-client ", quoted_uri, NULL);
+		g_free (quoted_uri);
 		anjuta_util_execute_shell (NULL, cmd);
 		g_free (cmd);
 		return;
