@@ -68,8 +68,8 @@ static gboolean find_incremental(IAnjutaEditor* te, gchar* expression,
 	if (info != NULL)
 	{
 		IAnjutaIterable *start, *end;
-		start = ianjuta_editor_get_cell_iter (te, info->pos, NULL);
-		end = ianjuta_editor_get_cell_iter (te, info->pos + info->len, NULL);
+		start = ianjuta_editor_get_position_from_offset (te, info->pos, NULL);
+		end = ianjuta_editor_get_position_from_offset (te, info->pos + info->len, NULL);
 		ianjuta_editor_selection_set (IANJUTA_EDITOR_SELECTION (te),
 									  start, end, NULL);
 		g_object_unref (start);
@@ -205,8 +205,8 @@ static GtkActionEntry actions_search[] = {
 	"<control>h",
 	N_("Search for and replace a string or regular expression with another string"),
     G_CALLBACK (on_find_and_replace1_activate)},
-  { "ActionEditAdvancedSearch", GTK_STOCK_FIND, N_("Advanced Search And Replace"),
-	NULL, N_("New advance search And replace stuff"),
+  { "ActionEditAdvancedSearch", GTK_STOCK_FIND, N_("Search and Replace"),
+	NULL, N_("Search and Replace"),
 	G_CALLBACK (on_find1_activate)},
   { "ActionEditSearchInFiles", NULL, N_("Fin_d in Files..."), NULL,
 	N_("Search for a string in multiple files or directories"),
