@@ -18,8 +18,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, 
- * Boston, MA 02111-1307, USA. 
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, 
+ * Boston, MA 02110-1301, USA. 
  */
  
 /*
@@ -27,7 +27,7 @@
  * list of people on the anjuta Team.  
  * See the ChangeLog files for a list of changes.
  *
- * $Id: anjuta-view.c,v 1.11 2006/05/15 13:25:41 jhs Exp $ 
+ * $Id: anjuta-view.c 3766 2008-03-11 14:05:05Z jhs $ 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -40,6 +40,7 @@
 #include <gdk/gdkkeysyms.h>
 
 #include <glib/gi18n.h>
+#include <glib.h>
 
 #include <libanjuta/anjuta-debug.h>
 
@@ -47,26 +48,12 @@
 #include "anjuta-encodings.h"
 #include "sourceview.h"
 #include "sourceview-private.h"
-#include "tag-window.h"
+#include "anjuta-marshal.h"
 
 #define ANJUTA_VIEW_SCROLL_MARGIN 0.02
 
 #define ANJUTA_VIEW_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), ANJUTA_TYPE_VIEW, AnjutaViewPrivate))
 
-enum {
-	CHAR_ADDED,
-	LAST_SIGNAL
-};
-
-static guint view_signals[LAST_SIGNAL] = { 0 };
-
-enum
-{
-	TAG = 0,
-	AUTOCOMPLETE,
-	SCOPE,
-};
-
 enum
 {
 	ANJUTA_VIEW_POPUP = 1
@@ -74,12 +61,9 @@ enum
 
 struct _AnjutaViewPrivate
 {
-	GtkTooltips *tooltips;
 	GtkWidget* popup;
 	guint scroll_idle;
-	
-	/* Tag and Autocompletion */
-	GList* tag_windows;
+  Sourceview* sv;
 };
 
 static void	anjuta_view_destroy		(GtkObject       *object);
@@ -127,52 +111,6 @@ document_read_only_notify_handler (AnjutaDocument *document,
 				    !anjuta_document_get_readonly (document));
 }
 
-/* Tag stuff */
-static TagWindow* get_active_tag_window(AnjutaView* view)
-{
-	GList* node = view->priv->tag_windows;
-	while (node)
-	{
-		if (tag_window_is_active(TAG_WINDOW(node->data)))
-			return TAG_WINDOW(node->data);
-		node = g_list_next(node);
-	}
-	return NULL;
-}
-
-static TagWindow* find_tag_window(AnjutaView* view, guint keyval)
-{
-	GList* node = view->priv->tag_windows;
-	while (node)
-	{
-		if (tag_window_filter_keypress(TAG_WINDOW(node->data), keyval)
-			== TAG_WINDOW_KEY_UPDATE)
-			return TAG_WINDOW(node->data);
-		node = g_list_next(node);
-	}
-	return NULL;
-}
-
-static void
-tag_window_selected(GtkWidget* window, gchar* tag_name, AnjutaView* view)
-{
-	GtkTextBuffer* buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
-	GtkTextIter cursor_iter, *word_iter;
-	gchar* current_word = anjuta_document_get_current_word(ANJUTA_DOCUMENT(buffer));
-	
-	gtk_text_buffer_get_iter_at_mark(buffer, &cursor_iter, 
-								 gtk_text_buffer_get_insert(buffer));
-	word_iter = gtk_text_iter_copy(&cursor_iter);
-	gtk_text_iter_set_line_index(word_iter, 
-		gtk_text_iter_get_line_index(&cursor_iter) - strlen(current_word));
-	gtk_text_buffer_delete(buffer, word_iter, &cursor_iter);
-	gtk_text_buffer_insert_at_cursor(buffer, tag_name, 
-		strlen(tag_name));
-		
-	g_free(current_word);
-	gtk_text_iter_free(word_iter);
-}
-
 static void
 anjuta_view_set_property (GObject * object,
 			   guint property_id,
@@ -262,17 +200,6 @@ anjuta_view_class_init (AnjutaViewClass *klass)
 					 ANJUTA_VIEW_POPUP,
 					 anjuta_view_spec_popup);
 	
-		view_signals[CHAR_ADDED] =
-   		g_signal_new ("char_added",
-			      G_OBJECT_CLASS_TYPE (object_class),
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (AnjutaViewClass, char_added),
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__CHAR,
-			      G_TYPE_NONE,
-			      1,
-			      G_TYPE_CHAR);
-	
 	binding_set = gtk_binding_set_by_class (klass);	
 }
 
@@ -382,16 +309,15 @@ anjuta_view_init (AnjutaView *view)
 	 */
 		
 	g_object_set (G_OBJECT (view), 
-		      "wrap_mode", FALSE,
-		      "show_line_numbers", TRUE,
-		      "auto_indent", TRUE,
-		      "tabs_width", 4,
-		      "insert_spaces_instead_of_tabs", FALSE,
-		      "highlight_current_line", TRUE, 
-		      "smart_home_end", FALSE, /* Never changes this */
+		      "wrap-mode", FALSE,
+		      "show-line-numbers", TRUE,
+		      "auto-indent", TRUE,
+		      "tab-width", 4,
+		      "insert-spaces-instead-of-tabs", FALSE,
+		      "highlight-current-line", TRUE, 
+          "indent-on-tab", TRUE, /* Fix #388727 */
+		      "smart-home-end", FALSE, /* Never changes this */
 		      NULL);
-	
-	view->priv->tag_windows = NULL;
 }
 
 static void
@@ -410,11 +336,13 @@ anjuta_view_finalize (GObject *object)
 	AnjutaView *view;
 
 	view = ANJUTA_VIEW (object);
-	
-	if (view->priv->tooltips != NULL)
-		g_object_unref (view->priv->tooltips);
-	
-	g_list_free(view->priv->tag_windows);
+
+	if (view->priv->popup != NULL) 
+	{
+	    GtkWidget* widget = gtk_menu_get_attach_widget (GTK_MENU (view->priv->popup));
+	    if (widget != NULL)
+		gtk_menu_detach (GTK_MENU (view->priv->popup));
+	}
 		
 	(* G_OBJECT_CLASS (anjuta_view_parent_class)->finalize) (object);
 }
@@ -423,14 +351,18 @@ static gint
 anjuta_view_focus_out (GtkWidget *widget, GdkEventFocus *event)
 {
 	AnjutaView *view = ANJUTA_VIEW (widget);
-	TagWindow* tag_window = get_active_tag_window(view);
-	if (tag_window != NULL)
-	{
-		gtk_widget_hide(GTK_WIDGET(tag_window));
-	}
+	AssistWindow* assist_win = view->priv->sv->priv->assist_win;
+	AssistTip* assist_tip = view->priv->sv->priv->assist_tip;
+	
+	if (assist_win)
+		gtk_widget_destroy(GTK_WIDGET(assist_win));
+	
+	if (assist_tip)
+		gtk_widget_destroy(GTK_WIDGET(assist_tip));
 	
 	gtk_widget_queue_draw (widget);
 	
+	
 	(* GTK_WIDGET_CLASS (anjuta_view_parent_class)->focus_out_event) (widget, event);
 	
 	return FALSE;
@@ -439,35 +371,37 @@ anjuta_view_focus_out (GtkWidget *widget, GdkEventFocus *event)
 
 /**
  * anjuta_view_new:
- * @doc: a #AnjutaDocument
+ * @sv: a #Sourceview
  * 
- * Creates a new #AnjutaView object displaying the @doc document. 
+ * Creates a new #AnjutaView object displaying the @sv->priv->doc document. 
  * @doc cannot be NULL.
  *
  * Return value: a new #AnjutaView
  **/
 GtkWidget *
-anjuta_view_new (AnjutaDocument *doc)
+anjuta_view_new (Sourceview *sv)
 {
 	GtkWidget *view;
 
-	g_return_val_if_fail (ANJUTA_IS_DOCUMENT (doc), NULL);
+	g_return_val_if_fail (ANJUTA_IS_DOCUMENT (sv->priv->document), NULL);
 
 	view = GTK_WIDGET (g_object_new (ANJUTA_TYPE_VIEW, NULL));
 	
 	gtk_text_view_set_buffer (GTK_TEXT_VIEW (view),
-				  GTK_TEXT_BUFFER (doc));
+				  GTK_TEXT_BUFFER (sv->priv->document));
   		
-	g_signal_connect (doc,
+	g_signal_connect (sv->priv->document,
 			  "notify::read-only",
 			  G_CALLBACK (document_read_only_notify_handler),
 			  view);
 
 	gtk_text_view_set_editable (GTK_TEXT_VIEW (view), 
-				    !anjuta_document_get_readonly (doc));					  
+				    !anjuta_document_get_readonly (sv->priv->document));					  
 
 	gtk_widget_show_all (view);
 
+  ANJUTA_VIEW(view)->priv->sv = sv;
+  
 	return view;
 }
 
@@ -722,9 +656,6 @@ anjuta_view_set_colors (AnjutaView *view,
 		rc_style->color_flags [GTK_STATE_ACTIVE] = 0;
 
 		gtk_widget_modify_style (GTK_WIDGET (view), rc_style);
-
-		/* It must be called after the text color has been modified */
-		modify_cursor_color (GTK_WIDGET (view), NULL);
 	}
 }
 
@@ -764,14 +695,6 @@ anjuta_view_set_font (AnjutaView   *view,
 	}
 }
 
-void anjuta_view_register_completion(AnjutaView* view, TagWindow* tagwin)
-{
-	g_return_if_fail(tagwin != NULL);
-	
-	view->priv->tag_windows = g_list_append(view->priv->tag_windows, tagwin);
-	g_signal_connect(G_OBJECT(tagwin), "selected", G_CALLBACK(tag_window_selected), view);
-}
-
 static gint
 anjuta_view_expose (GtkWidget      *widget,
                    GdkEventExpose *event)
@@ -800,76 +723,59 @@ anjuta_view_expose (GtkWidget      *widget,
 	return (* GTK_WIDGET_CLASS (anjuta_view_parent_class)->expose_event)(widget, event);
 }
 
-static gchar
-anjuta_view_get_last_character(GtkWidget* widget)
-{
-	GtkTextBuffer* buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
-	GtkTextIter cursor;
-	
-	gtk_text_buffer_get_iter_at_mark(buffer, &cursor, gtk_text_buffer_get_insert(buffer));
-	return (gchar) gtk_text_iter_get_char(&cursor);
-}
-
 static gboolean
 anjuta_view_key_press_event		(GtkWidget *widget, GdkEventKey       *event)
 {
+	GtkTextBuffer *buffer;
 	AnjutaView* view = ANJUTA_VIEW(widget);
-	TagWindow* tag_window = get_active_tag_window(view);
+	AssistWindow* assist_win;
+	AssistTip* assist_tip;
 	
-	 switch (event->keyval)
-	 {
-		case GDK_Shift_L:
-		case GDK_Shift_R:
+	buffer  = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
+	
+	assist_win = view->priv->sv->priv->assist_win;
+	if (assist_win)
+	{
+		if (assist_window_filter_keypress(assist_win, event->keyval))
 		{
+			DEBUG_PRINT("key filtered: %d", event->keyval);
 			return TRUE;
 		}
-		default:
-		{
-			gboolean retval;
-			if (tag_window != NULL)
-			{
-				switch (tag_window_filter_keypress(tag_window, event->keyval))
-				{
-					case TAG_WINDOW_KEY_UPDATE:
-					{
-						retval = (* GTK_WIDGET_CLASS (anjuta_view_parent_class)->key_press_event)(widget, event);
-						tag_window_update(tag_window, GTK_WIDGET(view));
-						g_signal_emit_by_name(G_OBJECT(view), "char_added", anjuta_view_get_last_character(widget));
-						return retval;
-					}
-					case TAG_WINDOW_KEY_CONTROL:
-					{
-						return TRUE;
-					}
-					case TAG_WINDOW_KEY_SKIP:
-					{
-						gtk_widget_hide(GTK_WIDGET(tag_window));
-					}
-				}
-			}
-			if ((tag_window = find_tag_window(view, event->keyval)) != NULL)
-			{
-				retval = (* GTK_WIDGET_CLASS (anjuta_view_parent_class)->key_press_event)(widget, event);
-				tag_window_update(tag_window, GTK_WIDGET(view));
-				g_signal_emit_by_name(G_OBJECT(view), "char_added", anjuta_view_get_last_character(widget));
-				return retval;
-			}
-			retval = (* GTK_WIDGET_CLASS (anjuta_view_parent_class)->key_press_event)(widget, event);
-			g_signal_emit_by_name(G_OBJECT(view), "char_added", anjuta_view_get_last_character(widget));
-			return retval;
-		}				
 	}
+	assist_tip = view->priv->sv->priv->assist_tip;
+	if (assist_tip)
+	{
+    switch (event->keyval)
+    {
+      case GDK_Escape:
+      case GDK_Up:
+      case GDK_Down:
+      case GDK_Page_Up:
+      case GDK_Page_Down:
+        gtk_widget_destroy (GTK_WIDGET(assist_tip));
+        break;
+		}
+	}
+	return (* GTK_WIDGET_CLASS (anjuta_view_parent_class)->key_press_event)(widget, event);
 }
 
-static gboolean	anjuta_view_button_press_event		(GtkWidget         *widget, GdkEventButton       *event)
+static gboolean	
+anjuta_view_button_press_event	(GtkWidget *widget, GdkEventButton *event)
 {
 	AnjutaView* view = ANJUTA_VIEW(widget);
-	TagWindow* tag_window = get_active_tag_window(view);
-	if (tag_window != NULL)
-	{
-		gtk_widget_hide(GTK_WIDGET(tag_window));
-	}
 	
+  /* If we have a calltip shown - hide it */
+  AssistTip* assist_tip = view->priv->sv->priv->assist_tip;
+  AssistWindow* assist_win = view->priv->sv->priv->assist_win;
+  if (assist_win)
+  {
+    gtk_widget_destroy (GTK_WIDGET (assist_win));
+  }
+	if (assist_tip)
+	{
+    gtk_widget_destroy (GTK_WIDGET (assist_tip));
+  }
+  
 	switch(event->button)
 	{
 		case 3: /* Right Button */
