@@ -50,9 +50,6 @@
 #define TIMEOUT_INTERVAL_SYMBOLS_UPDATE		10000
 
 static gpointer parent_class = NULL;
-static gboolean need_symbols_update;
-static gint timeout_id;
-static gchar prev_char_added = ' ';
 
 /* these will block signals on treeview and treesearch callbacks functions */
 static void trees_signals_block (SymbolBrowserPlugin *sv_plugin);
@@ -65,9 +62,6 @@ static void update_editor_symbol_model (SymbolBrowserPlugin *sv_plugin);
 
 static void on_editor_update_ui (IAnjutaEditor *editor,
 								 SymbolBrowserPlugin *sv_plugin);
-static void on_char_added (IAnjutaEditor *editor, gint position, gchar ch,
-						   SymbolBrowserPlugin *sv_plugin);
-
 static void
 register_stock_icons (AnjutaPlugin *plugin)
 {
@@ -87,16 +81,16 @@ static void
 goto_file_line (AnjutaPlugin *plugin, const gchar *filename, gint lineno)
 {
 	gchar *uri;
-	IAnjutaFileLoader *loader;
+	IAnjutaDocumentManager *docman;
 	
 	g_return_if_fail (filename != NULL);
 		
 	/* Go to file and line number */
-	loader = anjuta_shell_get_interface (plugin->shell, IAnjutaFileLoader,
+	docman = anjuta_shell_get_interface (plugin->shell, IAnjutaDocumentManager,
 										 NULL);
 		
-	uri = g_strdup_printf ("file:///%s#%d", filename, lineno);
-	ianjuta_file_loader_load (loader, uri, FALSE, NULL);
+	uri = gnome_vfs_get_uri_from_local_path (filename);
+	ianjuta_document_manager_goto_uri_line (docman, uri, lineno, NULL);
 	g_free (uri);
 }
 
@@ -695,9 +689,6 @@ on_editor_foreach_disconnect (gpointer key, gpointer value, gpointer user_data)
 	g_signal_handlers_disconnect_by_func (G_OBJECT(key),
 										  G_CALLBACK (on_editor_update_ui),
 										  user_data);
-	g_signal_handlers_disconnect_by_func (G_OBJECT(key),
-										  G_CALLBACK (on_char_added),
-										  user_data);
 	g_object_weak_unref (G_OBJECT(key),
 						 (GWeakNotify) (on_editor_destroy),
 						 user_data);
@@ -770,54 +761,6 @@ update_editor_symbol_model (SymbolBrowserPlugin *sv_plugin)
 	}
 }
 
-static gboolean
-on_editor_buffer_symbols_update_timeout (gpointer user_data)
-{
-	SymbolBrowserPlugin *sv_plugin;
-	IAnjutaEditor *ed;
-	gchar *current_buffer = NULL;
-	gint buffer_size = 0;
-	gchar *uri = NULL;
-
-	sv_plugin = ANJUTA_PLUGIN_SYMBOL_BROWSER (user_data);
-	
-	if (sv_plugin->current_editor == NULL)
-		return FALSE;
-	 
-	 /* we won't proceed with the updating of the symbols if we didn't type in 
-	 	anything */
-	 if (!need_symbols_update)
-	 	return TRUE;
-	
-	if (sv_plugin->current_editor) {
-		ed = IANJUTA_EDITOR (sv_plugin->current_editor);
-		
-		buffer_size = ianjuta_editor_get_length (ed, NULL);
-		current_buffer = ianjuta_editor_get_text (ed, 0, -1, NULL);
-				
-		uri = ianjuta_file_get_uri (IANJUTA_FILE (ed), NULL);
-		
-	} 
-	else
-		return FALSE;
-	
-	if (uri) {
-		/* FIXME: Only uncomment after investigating bug 395362 */
-		/*
-		anjuta_symbol_view_update_source_from_buffer (ANJUTA_SYMBOL_VIEW (sv_plugin->sv_tree), 
-				uri, current_buffer, buffer_size);
-		*/
-		g_free (uri);
-	}
-	
-	if (current_buffer)
-		g_free (current_buffer);  
-
-	need_symbols_update = FALSE;
-
-	return TRUE;
-}
-
 static gboolean
 iter_matches (SymbolBrowserPlugin *sv_plugin, GtkTreeIter* iter, 
 			  GtkTreeModel* model, gint lineno)
@@ -870,21 +813,6 @@ on_editor_update_ui (IAnjutaEditor *editor, SymbolBrowserPlugin *sv_plugin)
 	}
 }
 
-static void
-on_char_added (IAnjutaEditor *editor, gint position, gchar ch,
-			   SymbolBrowserPlugin *sv_plugin)
-{
-	DEBUG_PRINT ("char added @ %d : %c [int %d]", position, ch, ch);
-	
-	/* try to force the update if a "." or a "->" is pressed */
-	if ((ch == '.') || (prev_char_added == '-' && ch == '>'))
-		on_editor_buffer_symbols_update_timeout (sv_plugin);
-		
-	need_symbols_update = TRUE;
-	
-	prev_char_added = ch;
-}
-
 static void
 value_added_current_editor (AnjutaPlugin *plugin, const char *name,
 							const GValue *value, gpointer data)
@@ -929,21 +857,10 @@ value_added_current_editor (AnjutaPlugin *plugin, const char *name,
 		g_signal_connect (G_OBJECT (editor), "saved",
 						  G_CALLBACK (on_editor_saved),
 						  sv_plugin);
-						  
-		g_signal_connect (G_OBJECT (editor), "char-added",
-						  G_CALLBACK (on_char_added),
-						  sv_plugin);
 		g_signal_connect (G_OBJECT(editor), "update_ui",
 						  G_CALLBACK (on_editor_update_ui),
 						  sv_plugin);
 	}
-	
-	/* add a default timeout to the updating of buffer symbols */	
-	timeout_id = g_timeout_add (TIMEOUT_INTERVAL_SYMBOLS_UPDATE,
-								on_editor_buffer_symbols_update_timeout,
-								plugin);
-	need_symbols_update = FALSE;
-	
 }
 
 static void
@@ -955,8 +872,6 @@ value_removed_current_editor (AnjutaPlugin *plugin,
 	GtkAction *action;
 
 	/* let's remove the timeout for symbols refresh */
-	g_source_remove (timeout_id);
-	need_symbols_update = FALSE;
 	
 	sv_plugin = ANJUTA_PLUGIN_SYMBOL_BROWSER (plugin);
 	ui = anjuta_shell_get_ui (plugin->shell, NULL);
@@ -991,7 +906,7 @@ activate_plugin (AnjutaPlugin *plugin)
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sv_plugin->sl),
 									GTK_POLICY_AUTOMATIC,
 									GTK_POLICY_AUTOMATIC);
-	
+	/* Local symbols of the file */
 	sv_plugin->sl_tab_label = gtk_label_new (_("Local" ));
 	sv_plugin->sl_tree = anjuta_symbol_locals_new ();
 	g_object_add_weak_pointer (G_OBJECT (sv_plugin->sl_tree),
@@ -1011,7 +926,7 @@ activate_plugin (AnjutaPlugin *plugin)
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sv_plugin->sv),
 									GTK_POLICY_AUTOMATIC,
 									GTK_POLICY_AUTOMATIC);
-	
+	/* Global project-wide symbols */
 	sv_plugin->sv_tab_label = gtk_label_new (_("Global" ));
 	sv_plugin->sv_tree = anjuta_symbol_view_new ();
 	symbol_browser_load_global_tags (plugin);
@@ -1280,107 +1195,12 @@ isymbol_manager_get_parents (IAnjutaSymbolManager *sm,
 	return NULL;
 }
 
-static IAnjutaIterable*
-isymbol_manager_get_completions_at_position (IAnjutaSymbolManager *sm,
-											const gchar *file_uri,
-							 				const gchar *text_buffer, 
-											gint text_length, 
-											gint text_pos,
-							 				GError **err)
-{
-	SymbolBrowserPlugin *sv_plugin;
-	const TMTag *func_scope_tag;
-	TMSourceFile *tm_file;
-	IAnjutaEditor *ed;
-	AnjutaSymbolView *symbol_view;
-	gulong line;
-	gulong scope_position;
-	gchar *needed_text = NULL;
-	gint access_method;
-	GPtrArray * completable_tags_array;
-	AnjutaSymbolIter *iter = NULL;
-	
-	sv_plugin = ANJUTA_PLUGIN_SYMBOL_BROWSER (sm);
-	ed = IANJUTA_EDITOR (sv_plugin->current_editor);	
-	symbol_view = ANJUTA_SYMBOL_VIEW (sv_plugin->sv_tree);
-	
-	line = ianjuta_editor_get_line_from_position (ed, text_pos, NULL);
-	
-	/* get the function scope */
-	tm_file = anjuta_symbol_view_get_tm_file (symbol_view, file_uri);
-	
-	/* check whether the current file_uri is listed in the tm_workspace or not... */	
-	if (tm_file == NULL)
-		return 	NULL;
-		
-
-	// FIXME: remove DEBUG_PRINT	
-/*/
-	DEBUG_PRINT ("tags in file &s\n");
-	if (tm_file->work_object.tags_array != NULL) {
-		int i;
-		for (i=0; i < tm_file->work_object.tags_array->len; i++) {
-			TMTag *cur_tag;
-		
-			cur_tag = (TMTag*)g_ptr_array_index (tm_file->work_object.tags_array, i);
-			tm_tag_print (cur_tag, stdout);
-		}
-	}
-/*/
-
-	func_scope_tag = tm_get_current_function (tm_file->work_object.tags_array, line);
-		
-	if (func_scope_tag == NULL) {
-		DEBUG_PRINT ("func_scope_tag is NULL, seems like it's a completion on a global scope");
-		return NULL;
-	}
-	
-	DEBUG_PRINT ("current expression scope: %s", func_scope_tag->name);
-	
-	
-	scope_position = ianjuta_editor_get_line_begin_position (ed, func_scope_tag->atts.entry.line, NULL);
-	needed_text = ianjuta_editor_get_text (ed, scope_position,
-										   text_pos - scope_position, NULL);
-
-	if (needed_text == NULL)
-		DEBUG_PRINT ("needed_text is null");
-	DEBUG_PRINT ("text needed is %s", needed_text );
-	
-
-	/* we'll pass only the text of the current scope: i.e. only the current function
-	 * in which we request the completion. */
-	TMTag * found_type = anjuta_symbol_view_get_type_of_expression (symbol_view, 
-				needed_text, text_pos - scope_position, func_scope_tag, &access_method);
-
-				
-	if (found_type == NULL) {
-		DEBUG_PRINT ("type not found.");
-		return NULL;	
-	}
-	
-	/* get the completable memebers. If the access is COMPLETION_ACCESS_STATIC we don't
-	 * want to know the parents members of the class.
-	 */
-	if (access_method == COMPLETION_ACCESS_STATIC)
-		completable_tags_array = anjuta_symbol_view_get_completable_members (found_type, FALSE);
-	else
-		completable_tags_array = anjuta_symbol_view_get_completable_members (found_type, TRUE);
-	
-	if (completable_tags_array && completable_tags_array->len)
-	{
-		iter = anjuta_symbol_iter_new (completable_tags_array);
-		return IANJUTA_ITERABLE (iter);
-	}
-	return NULL;
-}
-
 static void
 isymbol_manager_iface_init (IAnjutaSymbolManagerIface *iface)
 {
 	iface->search = isymbol_manager_search;
 	iface->get_members = isymbol_manager_get_members;
 	iface->get_parents = isymbol_manager_get_parents;
-	iface->get_completions_at_position = isymbol_manager_get_completions_at_position;
 }
 
 static void
