@@ -88,8 +88,8 @@ struct _TerminalPlugin{
 	GtkWidget *scrollbar;
 	GtkWidget *pref_profile_combo;
 	GtkWidget *pref_default_button;
+	gboolean   widget_added_to_shell;
 	GList *gconf_notify_ids;
-	gboolean child_initizlized;
 #if OLD_VTE == 1
 	gboolean first_time_realization;
 #endif
@@ -396,8 +396,9 @@ terminal_execute (TerminalPlugin *term_plugin, const gchar *directory,
 	
 	preferences_changed (term_plugin->prefs, term_plugin);
 	
-	anjuta_shell_present_widget (ANJUTA_PLUGIN (term_plugin)->shell,
-								 term_plugin->frame, NULL);
+	if (term_plugin->widget_added_to_shell)
+		anjuta_shell_present_widget (ANJUTA_PLUGIN (term_plugin)->shell,
+									 term_plugin->frame, NULL);
 	
 	return term_plugin->child_pid;
 }
@@ -424,13 +425,9 @@ static gboolean
 terminal_focus_cb (GtkWidget *widget, GdkEvent  *event,
 				   TerminalPlugin *term) 
 {
-	if (term->child_pid > 0)
-		term->child_initizlized = TRUE;
-	
-	if (term->child_initizlized == FALSE)
+	if (term->child_pid == 0)
 	{
 		terminal_init_cb (widget, term);
-		term->child_initizlized = TRUE;
 	}
 	gtk_widget_grab_focus (widget);
 	return FALSE;
@@ -563,6 +560,8 @@ terminal_create (TerminalPlugin *term_plugin)
 	term_plugin->scrollbar = sb;
 	term_plugin->frame = frame;
 	term_plugin->hbox = hbox;
+
+	terminal_init_cb (GTK_WIDGET (term_plugin->term), term_plugin);
 }
 
 static void
@@ -590,7 +589,7 @@ activate_plugin (AnjutaPlugin *plugin)
 	term_plugin = ANJUTA_PLUGIN_TERMINAL (plugin);
 	term_plugin->ui = anjuta_shell_get_ui (plugin->shell, NULL);
 	term_plugin->prefs = anjuta_shell_get_preferences (plugin->shell, NULL);
-	
+	term_plugin->widget_added_to_shell = FALSE;
 	terminal_create (term_plugin);
 	
 	if (!initialized)
@@ -607,7 +606,7 @@ activate_plugin (AnjutaPlugin *plugin)
 							 "terminal-plugin-icon",
 							 ANJUTA_SHELL_PLACEMENT_BOTTOM, NULL);
 	/* terminal_focus_cb (term_plugin->term, NULL, term_plugin); */
-	
+	term_plugin->widget_added_to_shell = TRUE;
 	initialized = TRUE;
 	return TRUE;
 }
@@ -637,7 +636,7 @@ deactivate_plugin (AnjutaPlugin *plugin)
 	term_plugin->term = NULL;
 	term_plugin->scrollbar = NULL;
 	term_plugin->hbox = NULL;
-	term_plugin->child_initizlized = FALSE;
+	term_plugin->child_pid = 0;
 #if OLD_VTE == 1
 	term_plugin->first_time_realization = TRUE;
 #endif
@@ -663,7 +662,7 @@ terminal_plugin_instance_init (GObject *obj)
 {
 	TerminalPlugin *plugin = ANJUTA_PLUGIN_TERMINAL (obj);
 	plugin->gconf_notify_ids = NULL;
-	plugin->child_initizlized = FALSE;
+	plugin->child_pid = 0;
 	plugin->pref_profile_combo = NULL;
 #if OLD_VTE == 1
 	plugin->first_time_realization = TRUE;
