@@ -17,8 +17,8 @@
  * You should have received a copy of the GNU General Public License
  * along with plugin.c.  See the file "COPYING".  If not,
  * write to:  The Free Software Foundation, Inc.,
- *            59 Temple Place - Suite 330,
- *            Boston,  MA  02111-1307, USA.
+ *            51 Franklin Street, Fifth Floor,
+ *            Boston,  MA  02110-1301, USA.
  */
 
 #include <config.h>
@@ -31,6 +31,7 @@
 #include <libanjuta/anjuta-debug.h>
 #include <libanjuta/interfaces/ianjuta-document-manager.h>
 #include <libanjuta/interfaces/ianjuta-project-manager.h>
+#include <libanjuta/interfaces/ianjuta-preferences.h>
 
 #include "vgdefaultview.h"
 #include "plugin.h"
@@ -39,13 +40,7 @@
 
 #define UI_FILE PACKAGE_DATA_DIR"/ui/anjuta-valgrind.ui"
 #define GLADE_FILE PACKAGE_DATA_DIR"/glade/anjuta-valgrind.glade"
-#define ICON_FILE PACKAGE_PIXMAPS_DIR"/anjuta-valgrind.png"
-
-#define REGISTER_ICON(icon, stock_id) \
-	pixbuf = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/"icon, NULL); \
-	icon_set = gtk_icon_set_new_from_pixbuf (pixbuf); \
-	gtk_icon_factory_add (icon_factory, stock_id, icon_set); \
-	g_object_unref (pixbuf);
+#define ICON_FILE "anjuta-valgrind-plugin-48.png"
 
 
 enum {
@@ -60,10 +55,6 @@ static gpointer parent_class;
 static void
 register_stock_icons (AnjutaPlugin *plugin)
 {
-	AnjutaUI *ui;
-	GtkIconFactory *icon_factory;
-	GtkIconSet *icon_set;
-	GdkPixbuf *pixbuf;
 	static gboolean registered = FALSE;
 
 	if (registered)
@@ -71,12 +62,11 @@ register_stock_icons (AnjutaPlugin *plugin)
 	registered = TRUE;
 
 	/* Register stock icons */
-	ui = anjuta_shell_get_ui (plugin->shell, NULL);
-	icon_factory = anjuta_ui_get_icon_factory (ui);
-	REGISTER_ICON ("anjuta-valgrind-knight.png", "valgrind-knight");
+	BEGIN_REGISTER_ICON (plugin);
+	REGISTER_ICON (ICON_FILE, "valgrind-knight");
+	END_REGISTER_ICON;
 }
 
-
 static void
 project_root_added (AnjutaPlugin *plugin, const gchar *name,
 					const GValue *value, gpointer user_data)
@@ -84,7 +74,7 @@ project_root_added (AnjutaPlugin *plugin, const gchar *name,
 	AnjutaValgrindPlugin *val_plugin;
 	const gchar *root_uri;
 
-	val_plugin = (AnjutaValgrindPlugin*) plugin;
+	val_plugin = ANJUTA_PLUGIN_VALGRIND (plugin);
 	root_uri = g_value_get_string (value);
 	
 	if (root_uri)
@@ -105,7 +95,7 @@ project_root_removed (AnjutaPlugin *plugin, const gchar *name,
 					  gpointer user_data)
 {
 	AnjutaValgrindPlugin *val_plugin;
-	val_plugin = (AnjutaValgrindPlugin*) plugin;
+	val_plugin = ANJUTA_PLUGIN_VALGRIND (plugin);
 	
 	if (val_plugin->project_root_uri)
 		g_free(val_plugin->project_root_uri);
@@ -175,7 +165,7 @@ on_options_button_clicked (GtkButton *button, GladeXML *gxml)
 static void
 on_menu_editrules_activate (GtkAction *action, AnjutaValgrindPlugin *plugin)  
 {
-	vg_tool_view_show_rules ((VgToolView *) plugin->valgrind_widget);
+	vg_tool_view_show_rules (VG_TOOL_VIEW (plugin->valgrind_widget));
 }
 
 static void
@@ -199,7 +189,7 @@ on_menu_run_activate (GtkAction *action, AnjutaValgrindPlugin *plugin)
 						 IANJUTA_PROJECT_MANAGER_TARGET_EXECUTABLE,
 						 NULL);
 
-	if (g_list_length (exec_targets) >= 1) {
+	if (exec_targets) {
 		GladeXML *gxml;
 		GtkWidget *dlg, *treeview, *tool_combobox;
 		GtkTreeViewColumn *column;
@@ -209,7 +199,9 @@ on_menu_run_activate (GtkAction *action, AnjutaValgrindPlugin *plugin)
 		gint response, tool_selected;
 		GList *node;
 		GtkTreeIter iter;
-		const gchar *sel_target = NULL;
+		gchar *project_root_uri;
+		size_t project_root_uri_len;
+		gchar *sel_target = NULL;
 
 		tool_selected = 0;
 		gxml = glade_xml_new (GLADE_FILE, "select_and_run_dialog",
@@ -230,11 +222,15 @@ on_menu_run_activate (GtkAction *action, AnjutaValgrindPlugin *plugin)
 					  GTK_WINDOW (ANJUTA_PLUGIN(plugin)->shell));
 		store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
 		node = exec_targets;	
+
+		anjuta_shell_get (ANJUTA_PLUGIN (plugin)->shell,
+			"project_root_uri", G_TYPE_STRING,
+			&project_root_uri, NULL);
+		project_root_uri_len = strlen(project_root_uri) + 1;
 	
 		while (node) {
 			const gchar *rel_path;
-			rel_path = 	(gchar*)node->data + 
-					strlen (plugin->project_root_uri) + 1;
+			rel_path = (gchar*)node->data + project_root_uri_len;
 			gtk_list_store_append (store, &iter);
 			gtk_list_store_set (store, &iter, 0, rel_path, 1,
 								node->data, -1);
@@ -282,6 +278,7 @@ on_menu_run_activate (GtkAction *action, AnjutaValgrindPlugin *plugin)
 			gchar *prgname;
 			gchar *program_dir;
 			SymTab *symtab;
+			VgToolView *vg_tool_view;
 
 			prgname = gnome_vfs_format_uri_for_display (sel_target);
 			DEBUG_PRINT ("target program selected is %s", prgname);			
@@ -290,19 +287,20 @@ on_menu_run_activate (GtkAction *action, AnjutaValgrindPlugin *plugin)
 			program_dir = g_path_get_dirname (prgname);
 			DEBUG_PRINT ("target a basedir: %s", program_dir);
 			
-			vg_tool_view_set_argv ((VgToolView *) plugin->valgrind_widget, 
-							(const char**)sel_target);
-			vg_tool_view_set_srcdir ((VgToolView *) plugin->valgrind_widget, 
-							(const char**)program_dir);
+			vg_tool_view = VG_TOOL_VIEW (plugin->valgrind_widget);
+			
+			vg_tool_view_set_argv (vg_tool_view, sel_target, NULL);
+			vg_tool_view_set_srcdir (vg_tool_view, program_dir, NULL);
+			
+			g_free (sel_target);
+			g_free (program_dir);
 			
 			symtab = load_symtab (prgname);
-			vg_tool_view_set_symtab ((VgToolView *) plugin->valgrind_widget,
-							symtab);			
-
-		
+			vg_tool_view_set_symtab (vg_tool_view, symtab);			
+			
 			if (plugin->valgrind_displayed == FALSE) {
 				/* Add the widget to the shell */		
-				anjuta_shell_add_widget (((AnjutaPlugin*)plugin)->shell, plugin->valgrind_widget,
+				anjuta_shell_add_widget (ANJUTA_PLUGIN (plugin)->shell, plugin->valgrind_widget,
 								 "AnjutaValgrindPluginWidget", _("Valgrind"), "valgrind-knight",
 								 ANJUTA_SHELL_PLACEMENT_BOTTOM, NULL);	
 				plugin->valgrind_displayed = TRUE;
@@ -331,7 +329,7 @@ on_menu_run_activate (GtkAction *action, AnjutaValgrindPlugin *plugin)
 		else {
 			if ( response == GTK_RESPONSE_OK ) {
 				anjuta_util_dialog_error (GTK_WINDOW (ANJUTA_PLUGIN (plugin)->shell),
-						_("No executable target selected for debugging with Valgrind."
+						_("No executable target selected for debugging with Valgrind. "
 						"Please select one."));
 			}
 		}
@@ -423,7 +421,7 @@ on_menu_load_log_activate (GtkAction *action, AnjutaValgrindPlugin *plugin)
 
 	if (plugin->valgrind_displayed == FALSE) {
 		/* Add the widget to the shell */		
-		anjuta_shell_add_widget (((AnjutaPlugin*)plugin)->shell, plugin->valgrind_widget,
+		anjuta_shell_add_widget (ANJUTA_PLUGIN (plugin)->shell, plugin->valgrind_widget,
 						 "AnjutaValgrindPluginWidget", _("Valgrind"), "valgrind-knight",
 						 ANJUTA_SHELL_PLACEMENT_BOTTOM, NULL);	
 		plugin->valgrind_displayed = TRUE;
@@ -452,7 +450,7 @@ static GtkActionEntry actions_file[] = {
 	{
 		"ActionValgrindRun",                    /* Action name */
 		GTK_STOCK_EXECUTE,                      /* Stock icon, if any */
-		N_("_Select tool and run..."),			/* Display label */
+		N_("_Select Tool and Run..."),			/* Display label */
 		NULL,                                   /* short-cut */
 		NULL,                      				/* Tooltip */			
 		G_CALLBACK(on_menu_run_activate)		/* action callback */
@@ -460,23 +458,23 @@ static GtkActionEntry actions_file[] = {
 	{
 		"ActionValgrindKill",                   /* Action name */
 		GTK_STOCK_CANCEL,                      	/* Stock icon, if any */
-		N_("_Kill execution"),					/* Display label */
+		N_("_Kill Execution"),					/* Display label */
 		NULL,                                   /* short-cut */
 		NULL,                      				/* Tooltip */			
 		G_CALLBACK(on_menu_kill_activate)		/* action callback */
 	},
 	{
 		"ActionValgrindLoad",                   /* Action name */
-		GTK_STOCK_OPEN,                      	/* Stock icon, if any */
-		N_("_Load log"),						/* Display label */
+		NULL,                      	            /* Stock icon, if any */
+		N_("_Load Log"),						/* Display label */
 		NULL,                                   /* short-cut */
 		NULL,                      				/* Tooltip */			
 		G_CALLBACK(on_menu_load_log_activate)	/* action callback */
 	},
 	{
 		"ActionValgrindSave",                   /* Action name */
-		GTK_STOCK_SAVE,                      	/* Stock icon, if any */
-		N_("S_ave log"),						/* Display label */
+		NULL,                               	/* Stock icon, if any */
+		N_("S_ave Log"),						/* Display label */
 		NULL,                                   /* short-cut */
 		NULL,                      				/* Tooltip */
 		G_CALLBACK(on_menu_save_log_activate)	/* action callback */
@@ -534,15 +532,12 @@ valgrind_update_ui (AnjutaValgrindPlugin *plugin)
 static gboolean
 valgrind_activate (AnjutaPlugin *plugin)
 {
-	AnjutaPreferences *prefs;
 	AnjutaUI *ui;
-	GdkPixbuf *pixbuf;
 	static gboolean initialized = FALSE;
 	AnjutaValgrindPlugin *valgrind;
-	ValgrindPluginPrefs *valgrind_prefs;	
 	
 	DEBUG_PRINT ("AnjutaValgrindPlugin: Activating AnjutaValgrindPlugin plugin ...");
-	valgrind = (AnjutaValgrindPlugin*) plugin;
+	valgrind = ANJUTA_PLUGIN_VALGRIND (plugin);
 
 	if (!initialized) {
 		register_stock_icons (plugin);
@@ -550,36 +545,21 @@ valgrind_activate (AnjutaPlugin *plugin)
 
 	/* Add all UI actions and merge UI */
 	ui = anjuta_shell_get_ui (plugin->shell, NULL);
-	anjuta_ui_add_action_group_entries (ui, "ActionGroupValgrind",
-										_("Use Valgrind debug tool"),
-										actions_file,
-										G_N_ELEMENTS (actions_file),
-										GETTEXT_PACKAGE, plugin);
+	valgrind->action_group =
+		anjuta_ui_add_action_group_entries (ui, "ActionGroupValgrind",
+											_("Use Valgrind debug tool"),
+											actions_file,
+											G_N_ELEMENTS (actions_file),
+											GETTEXT_PACKAGE, TRUE, plugin);
 	valgrind->uiid = anjuta_ui_merge (ui, UI_FILE);
 
-	/* Add prefs page */
-	valgrind_prefs = valgrind_plugin_prefs_new ();
-	valgrind->general_prefs = valgrind_plugin_prefs_get_anj_prefs ();
-	
-	prefs = anjuta_shell_get_preferences (plugin->shell, NULL);
-	if (!initialized) {
-		pixbuf = gdk_pixbuf_new_from_file (ICON_FILE, NULL);
-	
-		anjuta_preferences_dialog_add_page (ANJUTA_PREFERENCES_DIALOG (prefs),
-						"Valgrind", pixbuf, valgrind->general_prefs);
-		g_object_unref (pixbuf);
-	}
-
-	/* Build the ValgrindPluginPrefs */
-	valgrind->val_prefs = valgrind_plugin_prefs_new ();
-
 	/* Create the main valgrind widget [a VgToolView object...]. Do NOT add it now,
      * but only after a call to run () is made
 	 */
 	valgrind->valgrind_widget = vg_default_view_new (valgrind);
 	
 	/* And the VgActions */	
-	valgrind->val_actions = vg_actions_new (valgrind, valgrind->val_prefs, 
+	valgrind->val_actions = vg_actions_new (valgrind, &valgrind->val_prefs, 
 						valgrind->valgrind_widget);
 							 
 	/* set up project directory watch */
@@ -594,6 +574,10 @@ valgrind_activate (AnjutaPlugin *plugin)
 	valgrind_set_busy_status (valgrind, FALSE);
 	valgrind_update_ui (valgrind);
 
+	/* Create prefs */
+	valgrind->general_prefs = valgrind_plugin_prefs_get_anj_prefs ();
+	valgrind->val_prefs = valgrind_plugin_prefs_new ();
+	
 	initialized = TRUE;
 	return TRUE;
 }
@@ -604,26 +588,23 @@ valgrind_deactivate (AnjutaPlugin *plugin)
 	AnjutaUI *ui;
 	AnjutaValgrindPlugin* valgrind;
 	
-	valgrind = (AnjutaValgrindPlugin*)plugin;
+	valgrind = ANJUTA_PLUGIN_VALGRIND (plugin);
 	
 	DEBUG_PRINT ("AnjutaValgrindPlugin: Dectivating AnjutaValgrindPlugin plugin ...");
 
-	if ( valgrind->valgrind_widget != NULL ) {
-		anjuta_shell_remove_widget (plugin->shell, valgrind->valgrind_widget,
-									NULL);
+	if ( valgrind->valgrind_displayed ) {
 		valgrind->valgrind_displayed = FALSE;
+		anjuta_shell_remove_widget (plugin->shell, valgrind->valgrind_widget, NULL);
 	}
 
 	ui = anjuta_shell_get_ui (plugin->shell, NULL);
-	anjuta_ui_unmerge (ui, valgrind->uiid);
-
-
-	/* unref VgToolView object */
-	g_object_unref (valgrind->general_prefs);
-	g_object_unref (valgrind->val_actions);
 	
-	/* FIXME: what about the destroying of Anjuta's Preference page? */
-	//g_object_unref (valgrind->val_prefs);
+	anjuta_plugin_remove_watch (plugin, valgrind->root_watch_id, TRUE);
+	anjuta_ui_unmerge (ui, valgrind->uiid);
+	anjuta_ui_remove_action_group (ui, valgrind->action_group);
+	
+	/* unref VgToolView object */
+	g_object_unref (valgrind->val_actions);
 	
 	return TRUE;
 }
@@ -643,9 +624,9 @@ valgrind_dispose (GObject *obj)
 }
 
 static void
-valgrind_instance_init (GObject *obj)
+anjuta_valgrind_plugin_instance_init (GObject *obj)
 {
-	AnjutaValgrindPlugin *plugin = (AnjutaValgrindPlugin*)obj;
+	AnjutaValgrindPlugin *plugin = ANJUTA_PLUGIN_VALGRIND (obj);
 
 	plugin->uiid = 0;
 
@@ -655,7 +636,7 @@ valgrind_instance_init (GObject *obj)
 }
 
 static void
-valgrind_class_init (GObjectClass *klass) 
+anjuta_valgrind_plugin_class_init (GObjectClass *klass) 
 {
 	AnjutaPluginClass *plugin_class = ANJUTA_PLUGIN_CLASS (klass);
 
@@ -667,5 +648,39 @@ valgrind_class_init (GObjectClass *klass)
 	klass->dispose = valgrind_dispose;
 }
 
-ANJUTA_PLUGIN_BOILERPLATE (AnjutaValgrindPlugin, valgrind);
-ANJUTA_SIMPLE_PLUGIN (AnjutaValgrindPlugin, valgrind);
+static void 
+ipreferences_merge(IAnjutaPreferences* ipref, AnjutaPreferences* prefs, GError** e)
+{
+	GdkPixbuf* pixbuf;
+	AnjutaValgrindPlugin* valgrind = ANJUTA_PLUGIN_VALGRIND (ipref);
+
+	pixbuf = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/"ICON_FILE, NULL);
+
+	gtk_widget_show (valgrind->general_prefs);
+	anjuta_preferences_dialog_add_page (ANJUTA_PREFERENCES_DIALOG (anjuta_preferences_get_dialog (prefs)),
+						"Valgrind", _("Valgrind"), pixbuf, valgrind->general_prefs);
+	g_object_unref (pixbuf);
+}
+
+static void
+ipreferences_unmerge(IAnjutaPreferences* ipref, AnjutaPreferences* prefs, GError** e)
+{
+	AnjutaValgrindPlugin* valgrind = ANJUTA_PLUGIN_VALGRIND (ipref);
+	anjuta_preferences_dialog_remove_page(ANJUTA_PREFERENCES_DIALOG (anjuta_preferences_get_dialog (prefs)), 
+		_("Valgrind"));
+	/* Recreate object as it gets destroyed in the remove_page method */
+	valgrind->general_prefs = valgrind_plugin_prefs_get_anj_prefs ();
+}
+
+static void
+ipreferences_iface_init(IAnjutaPreferencesIface* iface)
+{
+	iface->merge = ipreferences_merge;
+	iface->unmerge = ipreferences_unmerge;	
+}
+
+ANJUTA_PLUGIN_BEGIN (AnjutaValgrindPlugin, anjuta_valgrind_plugin);
+ANJUTA_PLUGIN_ADD_INTERFACE(ipreferences, IANJUTA_TYPE_PREFERENCES);
+ANJUTA_PLUGIN_END;
+
+ANJUTA_SIMPLE_PLUGIN (AnjutaValgrindPlugin, anjuta_valgrind_plugin);
